// -------------------------------------------------------------
// Factored BRDF
// 
// Copyright (c) 2003 IMAGIRE Takashi. All rights reserved.
// -------------------------------------------------------------

// -------------------------------------------------------------
// O[oϐ
// -------------------------------------------------------------
float4x4 mWVP;
float4x4 mWV;
float4x4 mWV_IT;
float4 LightPos;
float4 EyePos;
float4 alpha;

// -------------------------------------------------------------
// eNX`
// -------------------------------------------------------------
texture MapP;
sampler SampP = sampler_state
{
    Texture = <MapP>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = NONE;

    AddressU = Clamp;
    AddressV = Clamp;
};
// -------------------------------------------------------------
texture MapQ;
sampler SampQ = sampler_state
{
    Texture = <MapQ>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = NONE;

    AddressU = Clamp;
    AddressV = Clamp;
};
// -------------------------------------------------------------
texture EnvMap;
sampler SampEnv = sampler_state
{
    Texture = <EnvMap>;
    MinFilter = POINT;
    MagFilter = POINT;
    MipFilter = NONE;

    AddressU = Clamp;
    AddressV = Clamp;
};
// -------------------------------------------------------------
// AvP[V璸_VF[_ɓnf[^
// -------------------------------------------------------------
struct VS_INPUT
{
    float4 Pos          : POSITION;
    float3 Normal       : NORMAL0;
    float2 Tex          : TEXCOORD0;
};
// -------------------------------------------------------------
// _VF[_sNZVF[_ɓnf[^
// -------------------------------------------------------------
struct VS_OUTPUT
{
    float4 Pos          : POSITION;
    float4 v            : TEXCOORD0;
    float4 h            : TEXCOORD1;
    float4 l            : TEXCOORD2;
    
    float3 Nv           : TEXCOORD3;
    
    float4 Intensity 	: COLOR0;
};
// -------------------------------------------------------------
// _VF[_sNZVF[_ɓnf[^
// -------------------------------------------------------------
struct PS_INPUT
{
    float4 Pos          : POSITION;
    float2 Tex          : TEXCOORD0;
};

// -------------------------------------------------------------
// Wϊ
// -------------------------------------------------------------
float4 Q(float3 In, float3 B, float3 T, float3 N)
{
	float4 Out = (float4)0;
	
	// \ʍWn֕ϊ
	float3 v;
	v.x = dot(B,In);
	v.y = dot(T,In);
	v.z = dot(N,In);
	
	// Rp[^Qp[^ɕϊ
	float pv = 7.0f/8.0f;
	float ar = 1.0/(2.0*(1.000000001+v.z));

	if(0.7*(8.0f/7.0f)<ar){
		// OɏóAOɂ
		Out = 0;
	}else if(0.5*(8.0f/7.0f)<ar){
		// r͐`
		Out = (0.7*(8.0f/7.0f)-ar)/(0.2*(8.0f/7.0f));
	}else{
		Out = 1;
	}
	
	Out.x = pv * ar * v.x + 0.5;
	Out.y = pv * ar * v.y + 0.5;
	
	return Out;
}
// -------------------------------------------------------------
// _VF[_vO
// -------------------------------------------------------------
VS_OUTPUT VS(VS_INPUT In)
{   
	VS_OUTPUT Out = (VS_OUTPUT)0;			 // o̓f[^
	
	// ʒuW
	Out.Pos = mul( In.Pos, mWVP );
	
	// \ʍWn̊xNg߂
	float3 N = In.Normal;					// @xNg
	float3 B = float3(0,1,0);				// ]@xNg
	float3 T = normalize(cross(N,B));		// ڃxNg
	B = cross(T,N);
	
	// KvȃxNgvZ
	float3 V = normalize(EyePos  -In.Pos.xyz);// _xNg
	float3 L = normalize(LightPos-In.Pos.xyz);// CgxNg
	float3 H = normalize(L+V);				// n[txNg

	// exNgeNX`W֕ϊ
	Out.v = Q(V, B,T,N);
	Out.l = Q(L, B,T,N);
	Out.h = Q(H, B,T,N);
	
	// Ƃ肠ACgɊւĔwʂ̏
	Out.Intensity = Out.l.a;

	// ʒuW
	Out.Nv = mul( In.Normal, mWV_IT );
	

    return Out;
}
// -------------------------------------------------------------
// sNZVF[_vO
// -------------------------------------------------------------
float4 PS(VS_OUTPUT In) : COLOR
{   
    float4 v = tex2D( SampP, In.v );
    float4 h = tex2D( SampP, In.h );
    float4 l = tex2D( SampQ, In.l );
    float4 gross = alpha * v * h * l * In.Intensity;
	
	// }bv
	float3 Nv = normalize(In.Nv);
	float3 Vv = float3(0,0,-1);
	float3 r = 2 * dot(Nv, Vv)*Nv - Vv;
	float  m = 0.5 * rsqrt(dot(Nv+Vv,Nv+Vv));// Ki萔
	float2 env_coord = {m * r.x + 0.5, - m * r.y + 0.5};
    float4 hdr = tex2D( SampEnv, env_coord );
	
	float F = 0.9*pow(1-dot(Nv,r), 4) + 0.1;// ␳ꂽtl
    
    // HDRLDRɕϊ
	float3 ldr = saturate( F * hdr.rgb * pow(2, 255*(hdr.a - 0.5) ));
	
	// ŏIIɍ
    return gross + float4(ldr,0);
}
// -------------------------------------------------------------
// HDRLDRɕϊsNZVF[_vO
// -------------------------------------------------------------
float4 HDR_PS(PS_INPUT In) : COLOR
{   
	float4 ldr = (float4)0;
	
    float4 hdr = tex2D( SampEnv, In.Tex );
	
	ldr.rgb = saturate(hdr * pow(2, 255*(hdr.a - 0.5) ));
	
    return ldr;
}
// -------------------------------------------------------------
// eNjbN
// -------------------------------------------------------------
technique TShader
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 PS();
    }
    pass P1
    {
        PixelShader  = compile ps_2_0 HDR_PS();
    }
}

